## Simulation Code for Monte Carlo CART Example 2 ## 

# Packages required for the simulation #

#install.packages('MASS')
#install.packages('lavaan')
#install.packages('PoisNonNor')
#install.packages('MBESS')

library(MASS)
library(lavaan)
library(PoisNonNor)
library(MBESS)

# Function to transform data through the Fleishman approach 
transform <- function(x,pmat){ pmat[1,1]+pmat[1,2]*x+pmat[1,3]*x^2+pmat[1,4]*x^3}

# Number of replications
t_rep <- 20

# Matrix to hold simulation results
p1 <- matrix(NA,ncol=13,nrow=t_rep)
colnames(p1) <- c('sample_size','reliability','items','fact_dist','sk',
'omega_est','omega_se','omega_lci','omega_uci','alpha_est','alpha_se','alpha_lci','alpha_uci')



#####################
##Simulation Starts##
#####################


# Fixed effects for skewness/kurtosis and factor loading distribution
nonnormality <- matrix(c(0,0,1.25,3.5,2.25,7),nrow=3,ncol=2,byrow=T)
facdist <- matrix(c(.2,.8,.5,.5),nrow=2,ncol=2,byrow=T)


# Simulation Loop

for (i in 1:t_rep){

# random factor sampling: reliability, number of items,and sample size
  rho <- runif(1,.6,.9)
  items <- sample(c(5:10),1)
  sample_size <- round(runif(1,100,1000),0)

# random sampling for fixed effects: ff=factor loading distribution, ss: skewness/kurtosis
  ff <- sample(1:2,1)
  ss <- sample(1:3,1)

# true values for factor loadings and residual variances
lambdas <- seq(facdist[ff,1],facdist[ff,2],length.out=items)
sum_lambda <- sum(lambdas)^2
residual <- round(((sum_lambda-sum_lambda*rho)/rho/items),3) #residual per latent variable

if(items==5){
model <-
'
f1 =~ lambda1*x1+lambda2*x2 + lambda3*x3+lambda4*x4+ lambda5*x5
x1 ~~ error*x1
x2 ~~ error*x2
x3 ~~ error*x3
x4 ~~ error*x4
x5 ~~ error*x5
f1 ~~ 1*f1'
}

if(items==6){
model <-
'
f1 =~ lambda1*x1+lambda2*x2 + lambda3*x3+lambda4*x4+ lambda5*x5+ lambda6*x6
x1 ~~ error*x1
x2 ~~ error*x2
x3 ~~ error*x3
x4 ~~ error*x4
x5 ~~ error*x5
x6 ~~ error*x6
f1 ~~ 1*f1'
}

if(items==7){
model <-
'
f1 =~ lambda1*x1+lambda2*x2 + lambda3*x3+
lambda4*x4+ lambda5*x5+ lambda6*x6 + lambda7*x7
x1 ~~ error*x1
x2 ~~ error*x2
x3 ~~ error*x3
x4 ~~ error*x4
x5 ~~ error*x5
x6 ~~ error*x6
x7 ~~ error*x7
f1 ~~ 1*f1'
}

if(items==8){
model <-
'
f1 =~ lambda1*x1+lambda2*x2 + lambda3*x3+
lambda4*x4+ lambda5*x5+ lambda6*x6 + lambda7*x7+ lambda8*x8
x1 ~~ error*x1
x2 ~~ error*x2
x3 ~~ error*x3
x4 ~~ error*x4
x5 ~~ error*x5
x6 ~~ error*x6
x7 ~~ error*x7
x8 ~~ error*x8
f1 ~~ 1*f1'
}

if(items==9){
model <-
'
f1 =~ lambda1*x1+lambda2*x2 + lambda3*x3+
lambda4*x4+ lambda5*x5+ lambda6*x6 + 
lambda7*x7+ lambda8*x8+lambda9*x9
x1 ~~ error*x1
x2 ~~ error*x2
x3 ~~ error*x3
x4 ~~ error*x4
x5 ~~ error*x5
x6 ~~ error*x6
x7 ~~ error*x7
x8 ~~ error*x8
x9 ~~ error*x9
f1 ~~ 1*f1'
}

if(items==10){
model <-
'
f1 =~ lambda1*x1+lambda2*x2 + lambda3*x3+lambda4*x4 + lambda5*x5+ 
lambda6*x6 +lambda7*x7 + lambda8*x8+lambda9*x9+ lambda10*x10

x1  ~~ error*x1
x2  ~~ error*x2
x3  ~~ error*x3
x4  ~~ error*x4
x5  ~~ error*x5
x6  ~~ error*x6
x7  ~~ error*x7
x8  ~~ error*x8
x9  ~~ error*x9
x10 ~~ error*x10
f1 ~~ 1*f1'
}

# Edit Lavaan Model based on true values 

for (j in 1:length(lambdas)){
 aa <- paste0("lambda",j)
 model <- gsub(aa, lambdas[j],model)
 model <- gsub("error",residual,model)

}


# Extract fitted covariance matrix and correlation matrix from model
true_cov  <- fitted(sem(model))$cov
true_corr <- cov2cor(true_cov)


# Transform data for non-normality
rmat = matrix(c(rep(c(nonnormality[ss,1],nonnormality[ss,2]),items)), byrow=T, ncol=2) 
pmat<-Param.fleishman(rmat)
int_cor<-intercor.NN(pmat, matrix(true_corr,nrow=items,ncol=items,byrow=T))


# Extract data from multivariate normal distribution and transform for nonnormality

dat2 <- mvrnorm(sample_size,rep(0,items),Sigma=int_cor)
dat3 <- sapply(dat2,transform,pmat=pmat)
dat4 <- matrix(dat3,ncol=items,byrow=F)


# Ask if raw datasets should be saved

if(i==1){
	n <- readline(prompt="Save Datasets (y/n): ")
}

if(n=='y'){

# Save raw datasets to the working directory 

write.table(dat4,file=paste0("rel",i,".dat"),row.names=F,col.names=F)
m1=ci.reliability(dat4,type='omega',interval.type="ml")[1:4]
m2=ci.reliability(dat4,type='alpha',interval.type='ml')[1:4]
p1[i,]<-c(sample_size,rho,items,ff,ss,m1[[1]],m1[[2]],m1[[3]],m1[[4]],m2[[1]],m2[[2]],m2[[3]],m2[[4]])
}

if(n=='n'){

# Datasets are not saved, but results are still recorded

m1=ci.reliability(dat2,type='omega',interval.type="ml")[1:4]
m2=ci.reliability(dat2,type='alpha',interval.type='ml')[1:4]
p1[i,]<-c(sample_size,rho,items,ff,ss,m1[[1]],m1[[2]],m1[[3]],m1[[4]],m2[[1]],m2[[2]],m2[[3]],m2[[4]])
}
}


# Saves the working results of the simulation

save(p1,file='mc.cartRelExample1.R')

# Turn data matrix into a data frame and calculate Monte Carlo outcomes
p2 <- as.data.frame(p1)


# recode fixed factors as factor

p2$fact_dist<-as.factor(p2$fact_dist)
p2$sk<-as.factor(p2$sk)


# compute Monte Carlo outcomes

p2$alpha_cov   <- ifelse(p2$reliability>=p2$alpha_lci & p2$reliability<=p2$alpha_uci,1,0)
p2$omega_cov   <- ifelse(p2$reliability>=p2$omega_lci & p2$reliability<=p2$omega_uci,1,0)
p2$alpha_width <- p2$alpha_uci-p2$alpha_lci 
p2$omega_width <- p2$omega_uci-p2$omega_lci 
p2$alpha_omega_diff <- p2$omega_est-p2$alpha_est
p2$rho_omega_diff   <- p2$reliability-p2$omega_est
p2$rho_alpha_diff   <- p2$reliability-p2$alpha_est
